; Locate X,Y Positionning

Cursor_X      equ 6      ; Start Position of the selection Cursor
Cursor_Y      equ 7
List_X        equ 7      ; Start Position of the list of games
List_Y        equ 7
Page_X        equ 12     ; Start Position of the Page Number
Page_Y        equ 19
Box_X         equ 4      ; Start position of the Box frame
Box_Y         equ 1
Title_X       equ 7      ; Where Title is displayed
Title_Y       equ 2

;ASCII 16KB MEGAROM address
_bank1			equ	0x6000
_bank2			equ	0x7000


SFX_Move           equ 0

;VRAM Addresses
sc1tnp         equ      #1800
sc1tc          equ      #2000
sc1tgp         equ      #0000

sc2tgp         equ      #0000
sc2tnp         equ      #1800
sc2sat         equ      #1b00
sc2tc          equ      #2000
sc2tgs         equ      #3800


ramworkarea    equ      endpt3ram+2
saveinterupt   equ      ramworkarea
currentsel     equ      saveinterupt+3
lastselpos     equ      currentsel+1
currentpage    equ      lastselpos+1
nbgames        equ      currentpage+1
pingpong       equ      nbgames+1
muteflag       equ      pingpong+1
muscnt         equ      muteflag+1
joy            equ      muscnt+1
joy_value      equ      joy+1
tasptr         equ      joy_value+1
tascntr        equ      tasptr+2
tgsptr         equ      tascntr+1
timer          equ      tgsptr+2
launch_program equ      timer+1
scroll_gfx     equ      launch_program

;MSX BIOS functions
dcompr: 	equ	#20	;compare hl et de
disscr:		equ	#41	;disable screen
enascr:		equ	#44	;enable screen
ldirmv		equ	#59	;vram -> ram
ldirvm		equ	#5c	;ram -> vram
chgmod		equ	#5f	;change graphic mode
chgclr		equ	#62	;change colors
clrspr		equ	#69	;clear sprites
init32          equ     #6f     ;set screen 1
beep		equ	#c0	;beep!!
cls		equ	#c3	;clear screen
gtstck		equ	#d5	;get joystick
gttrig		equ	#d8	;get trigger
gtpad		equ	#db	;get pad
snsmat          equ     #141    ;get keyboard Matrix
bigfil		equ	#16b	;fill vram
nsetrd		equ	#16e	;new set add for read
nstwrt		equ	#171	;new set add for write
nrdvrm		equ	#174	;new read vram
nwrvrm		equ	#177	;new write vram
chgcpu 	        equ	#180	;change cpu mode
getcpu          equ	#183	;get cpu mode

wrboot:		equ	#0000
bdos:		equ	#0005
rdslt:		equ	#000c
wrslt:		equ	#0014
CALSLT:		equ	#001c
enaslt:		equ	#0024
callf:		equ	#0030
setdma: 	equ     26
read:   	equ     39
open:   	equ     15
close:  	equ     16
subrom:		equ	#faf8		;sub rom slot
hokvld:		equ	#fb20
exttbl:		equ	#fcc1		;main rom slot
extbio:		equ	#ffca
clicksw         equ     #F3DB

;Common routines
; Mult h by e
; result in hl
mult8:
	ld l,0
	ld d,0
	ld b,8
mulloop:
	add hl,hl
	jr nc,suite
	add hl,de
suite:
	djnz mulloop
	ret

; Hex to BCD
; converts a hex number (eg. $10) to its BCD representation (eg. $16).
; Input: a = hex number
; Output: a = BCD number
; Clobbers: b,c
HexToBCD:
  ld c,a  ; Original (hex) number
   ld b,8  ; How many bits
   xor a   ; Output (BCD) number, starts at 0
HTBLOOP: sla c   ; shift c into carry
   adc a,a
   daa     ; Decimal adjust a, so shift = BCD x2 plus carry
   djnz HTBLOOP  ; Repeat for 8 bits
; Multiply DE by A and put the result in AHL
mult_a_de
   ld	c, 0
   ld	h, c
   ld	l, h

   add	a, a		; optimised 1st iteration
   jr	nc, $+4
   ld	h,d
   ld	l,e

   ld b, 7
_loop:
   add	hl, hl
   rla
   jr	nc, $+4
   add	hl, de
   adc	a, c            ; yes this is actually adc a, 0 but since c is free we set it to zero and so we can save 1 byte and up to 3 T-states per iteration

   djnz	_loop

   ret
testspace:
        ld a,8
        call snsmat
        and 1
        ret
testesc:
        ld a,7
        call snsmat
        and 4
        ret



; Simulate RST20 (compare hl,de) under dos
simrst20:
	ld a,h
	sub d
	ret nz
	ld a,l
	sub e
	ret
; Change freqency Pal / NTSC
chfreq:
	ld a,(#ffe8)
	xor 2
	ld (#ffe8),a
	out (#99),a
	ld a,128+9
	out (#99),a
	ret

; Set color palette
; in : hl points on the palette table
setpal:
       di
	xor a
	ld c,#9a
	out (#99),a
	ld a,128+16
	out (#99),a
	ld b,32
	otir
	ret

; Vpoke: position write address = de
; Only works in the first 64 kb of vram
vpoke:
      di
        ld a,e
        out (#99),a
        ld a,d
        and %00111111
        or %01000000
        out (#99),a
	ret



;
; Read Keyboard directions by Default. If no key pressed, will check joystick 1.
;
Read_Joystick_Keyboard:
        xor a
        call rd_stk
        cp 0
        ret nz
        ld a,1
        call rd_stk
        ret

;*******************************************************************
;
;	lecture des joystick (port 0, 1, 2) voir #d5 du bios
;
;*******************************************************************
rd_stk:	dec	a
	jp	m,keybrd	;saut si clavier (port 0)
	call	sel_stk		;selection du port joystick
	ld	hl,stick_tbl1
end_stk:
	and	#0f
	ld	e,a
	ld	d,0
	add	hl,de
	ld	a,(hl)
	or	a		;position le flag z si rien de nouveau ;-)
	ret

keybrd:	call	rd_stk0
	rrca
	rrca
	rrca
	rrca
	ld	hl,stick_tbl2
	jr	end_stk

sel_stk:			;selection du port joystick 1 ou 2
	ld	b,a		;numero de port dans b
	ld	a,#0f
	call	rd_psgport
	djnz	port2

        and	#df		;joystick port 1
        or	#03
        jp	rd_stick

port2   and	#af		;joystick port 2
	or	#03
rd_stick:
	out	(#a1),a
	call	rd_joy
	ret

rd_joy:
	ld	a,#0e
rd_psgport:
	out	(#a0),a
	in	a,(#a2)
	ret

rd_stk0:			;lit joystick port 0
	in	a,(#aa)
	and	#f0
	add	a,8
	out	(#aa),a
	in	a,(#a9)
	ret

stick_tbl1:
	defb	0,5,1,0,3,4,2,3,7,6,8,7,0,5,1,0
stick_tbl2:
	defb	0,3,5,4,1,2,0,3,7,0,6,5,8,1,7,0


;*********************************************************
;
;	lecture des switch fire voir #d8 du bios
;
;*********************************************************
rd_stg:
	dec	a
	jp	m,kbd_stg	;si strig 0 saute
	push	af
	and	1
	call	sel_stk
	pop	bc
	dec	b
	dec	b
	ld	b,#10
	jp	m,stg1
	ld	b,#20
stg1:	and	b
stg2:	sub	1
	sbc	a,a
	or	a		;positionne le flag z si pas d'appui
	ret

kbd_stg:
	call	rd_stk0
        and	1
	jr	stg2

displayoff:
           jp #41

displayon:
           jp #44

codeinit:
	call    0x138 ; Read primary slot register
	rrca
	rrca
	and     0x03  ; a = value for Page 1.
	ld      c,a
	ld      b,0
	ld      hl,0xfcc1
	add     hl,bc
	or      (hl)  ; (HL) = #80 = Slot 1 expanded, Slot 1 = not expanded.
	ld      b,a
	inc     hl
	inc     hl
	inc     hl
	inc     hl
	ld      a,(hl)
	and     0x0c
	or      b
	ld      h,0x80
	call    0x24

        ; make emulators detect correct romtype (scc
        jp .skipfakebankswitch
	ld a,0
	ld (_bank1),a
	ld a,1
	ld (_bank2),a
	ld a,2
	ld (_bank2),a
	ld a,3
	ld (_bank2),a
	ld a,4
	ld (_bank2),a
	ld a,5
	ld (_bank2),a
	ld a,6
	ld (_bank2),a
	ld a,7
	ld (_bank2),a

.skipfakebankswitch:
        ret

init_scroll:
      ; 0 = TGP 8 , 9
      ld hl,GFX+(8*8)
      ld de,scroll_gfx
      ld bc,16
      ldir

      ld hl,GFX+(8*8)
      ld bc,16
      ldir

      ; 32 = TGP 10 , 11
      ld hl,GFX+(10*8)
      ld bc,16
      ldir

      ld hl,GFX+(10*8)
      ld bc,16
      ldir

      ; 64 = TGP 12 , 13
      ld hl,GFX+(12*8)
      ld bc,16
      ldir

      ld hl,GFX+(12*8)
      ld bc,16
      ldir

      ; 96 = TGP 12 , 13
      ld hl,GFX+(14*8)
      ld bc,16
      ldir

      ld hl,GFX+(14*8)
      ld bc,16
      ldir

      ; 128 = TC 8 , 9
      ld hl,GFX+(96*8)+(8*8)
      ld bc,16
      ldir

      ld hl,GFX+(96*8)+(8*8)
      ld bc,16
      ldir

      ; 160 = tc 10,11
      ld hl,GFX+(96*8)+(10*8)
      ld bc,16
      ldir

      ld hl,GFX+(96*8)+(10*8)
      ld bc,16
      ldir

      ; 192 = tc 12,13
      ld hl,GFX+(96*8)+(12*8)
      ld bc,16
      ldir

      ld hl,GFX+(96*8)+(12*8)
      ld bc,16
      ldir

      ; 224 = tc 14,15
      ld hl,GFX+(96*8)+(14*8)
      ld bc,16
      ldir

      ld hl,GFX+(96*8)+(14*8)
      ld bc,16
      ldir

      ret

do_scroll:
      di
      ld de,sc2tgp+(8*8)
      call vpoke

      ld hl,scroll_gfx
      ld a,(timer)
      rrca
      and 15
      ld e,a
      ld d,0
      add hl,de
      ; 8,9
      ld b,16
      call slowotir
      ;10,11
      ld de,16
      add hl,de
      ld b,16
      call slowotir
      ;12,13
      ld de,16
      add hl,de
      ld b,16
      call slowotir
      ;14,15
      ld de,16
      add hl,de
      ld b,16
      call slowotir

      ld de,sc2tc+(8*8)
      call vpoke
      ld hl,scroll_gfx+128
      ld a,(timer)
      rrca
      and 15
      ld e,a
      ld d,0
      add hl,de
      ;8,9
      ld b,16
      call slowotir

      ld de,16
      add hl,de
      ;10,11
      ld b,16
      call slowotir

      ld de,16
      add hl,de
      ;12,13
      ld b,16
      call slowotir

      ld de,16
      add hl,de
      ;14,15
      ld b,16
      call slowotir

      ret
set_tc:
      di
      ld de,sc2tc
      call vpoke
      ld hl,GFX+(96*8)
      call load_tc

      ld de,sc2tc+(128*8)
      call vpoke
      ld b,0
load_tc2:
      ld a,#06
      out (#98),a
      nop
      nop
      nop
      out (#98),a
      nop
      nop
      nop
      out (#98),a
      nop
      nop
      nop
      djnz load_tc2

      ret
load_tc:
      di
      ld c,#98
      ld b,0
      call slowotir
      call slowotir
      call slowotir
      ret

set_tgp:
      di
      ld de,sc2tgp
      call vpoke
      ld hl,GFX
      call load_tgp

      ld de,sc2tgp+(128*8)    ; second set for alternative colors
      call vpoke
      ld hl,GFX
      call load_tgp
load_tgp:
	ld b,0
	call slowotir
        call slowotir
        call slowotir
        ret

load_tgp_char:
	ld b,8
	call slowotir
        ret

;  LOCATE X = H , Y = L
;
;
printathl:
        di
        ld a,l   ;       L= Y POSITIOn (0 to 23)
        rrca
        rrca
        rrca
        ld d,a
        and %11100000
        or h     ;       H = X Position (0 to 31)
        ld e,a   ;       lower part of address is in e

        ld a,d
        and %11  ;
        ld d,a   ;       Higher part of address is in H
        ld hl,sc1tnp
        add hl,de

        ld a,l
        out (#99),a
        ld a,h
        and %00111111
        or %01000000
        out (#99),a

print_loop:

        ld a,(bc)
        cp "!"
        ret z
        out (#98),a
        inc bc
        jp print_loop

slowotir:
         ld a,(#7)
         ld c,a
slowoutloop:
         outi
         jp nz,slowoutloop
         ret

outAtoVram:
           di
           exx
           ex af,af'
           ld a,(#7)
           ld c,a
           ex af,af'
           out (c),a
           exx
           ret


setbgr:
       di
       ld de,sc2tnp
       call vpoke
       ld b,12
setbgr_loop:
       push bc
       ld hl,pattern
       ld b,64
       call slowotir
       pop bc
       djnz setbgr_loop
       ret
pattern:
        db 8,10,8,10,8,10,8,10,8,10,8,10,8,10,8,10,8,10,8,10,8,10,8,10,8,10,8,10,8,10,8,10
        db 9,11,9,11,9,11,9,11,9,11,9,11,9,11,9,11,9,11,9,11,9,11,9,11,9,11,9,11,9,11,9,11

setbox:
       di
       ld h,Box_X
       ld l,Box_Y
       push hl
       ld bc,box0
       call printathl
       pop hl
       inc hl
       push hl
       ld bc,box0b
       call printathl
       pop hl
       inc hl
       push hl
       ld bc,box0c
       call printathl
       pop hl
       inc hl
       push hl
       ld bc,box0d
       call printathl
       pop hl
       inc hl
       push hl
       ld bc,box1
       call printathl
       pop hl
       inc l
       ld b,7
setbox_loop:
       push bc
       push hl
       ld bc,box2
       call printathl
       pop hl
       inc l
       push hl
       ld bc,box2b
       call printathl
       pop hl
       inc l
       pop bc
       djnz setbox_loop

       push hl
       ld bc,box3
       call printathl
       pop hl
       inc l
       ld bc,box4
       call printathl

       ret

box0:
       db 09,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,11,"!"
box0b:
       db 07,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,04,12,"!"
box0c:
       db 09,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,12,13,"!"
box0d:
       db 08,14,12,14,12,14,12,14,12,14,12,14,12,14,12,14,12,14,12,14,12,"!"


box1:
       db 09,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,06,11,"!"
box2:
       db 07,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,04,12,"!"
box2b:
       db 07,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,04,13,"!"
box3:
       db 08,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,14,12,"!"
box4:
       db 09,15,13,15,13,15,13,15,13,15,13,15,13,15,13,15,13,15,13,15,13,15,13,15,13,"!"




; ix = sprite structure
;
;  hl = xy
;  RAM USED
; (tasptr) = Current address of the sprite pointer
; (tascntr) = Counter to the sprites
; (tgsptr)
; (tcsptr)
; To DO : Check overflow of TAS
putsprite:
          di
           ld a,(ix)
           ld b,a
           inc ix
putsprite_loop:
           push bc
           push hl

            ld de,(tasptr)
            call vpoke
            inc de
            inc de
            inc de
            inc de
            ld (tasptr),de

            ld a,(ix)
            add a,l
            dec a
            out (#98),a
            ld a,(ix+1)
            add a,h
            call c,setoutofscreen
            out (#98),a
            ld a,(tascntr)
            nop
            out (#98),a
            add a,4
            ld (tascntr),a
            ld a,(ix+2)
            out (#98),a
            ld de,(tgsptr)
            call vpoke
            ld hl,32
            add hl,de
            ld (tgsptr),hl

            ld l,(ix+3)
            ld h,(ix+4)
            ld c,#98
            ld b,32
            call slowotir

            ld bc,5
            add ix,bc
            pop hl
            pop bc
            djnz putsprite_loop
            ld de,(tasptr)
            call vpoke

            ld a,208
            out (#98),a

            ret

setoutofscreen:
               ld a,255
               ret
initputsprite:
          ld de,sc2sat
          ld (tasptr),de
          ld de,sc2tgs
          ld (tgsptr),de
          xor a
          ld (tascntr),a
          ret


corners:
     db 8            ;1 sprites

     db 0,0          ;y,x offset
     db 14           ;color
     dw corner1      ;tgsadr

     db 32,0          ;y,x offset
     db 14           ;color
     dw corner1      ;tgsadr


     db 16,0        ;y,x offset
     db 14           ;color
     dw corner2      ;tgsadr

     db 152,0        ;y,x offset
     db 14           ;color
     dw corner2      ;tgsadr

     db 0,152        ;y,x offset
     db 14           ;color
     dw corner3      ;tgsadr

     db 32,184        ;y,x offset
     db 14           ;color
     dw corner3      ;tgsadr

     db 16,152      ;y,x offset
     db 14           ;color
     dw corner4      ;tgsadr

     db 152,184      ;y,x offset
     db 14           ;color
     dw corner4      ;tgsadr

     db 255          ; end of sprite

corner1:
        db %00001111
        db %00111111
        db %01111111
        db %01111111
        db %11111111
        db %11111111
        db %11111111
        db %11111111
        ds 48,0
corner2:
        db %11111111
        db %11111111
        db %11111111
        db %11111111
        db %01111111
        db %01111111
        db %00111111
        db %00001111
        ds 48,0
corner3:
        db %11110000
        db %11111100
        db %11111110
        db %11111110
        db %11111111
        db %11111111
        db %11111111
        db %11111111
        ds 48,0
corner4:
        db %11111111
        db %11111111
        db %11111111
        db %11111111
        db %11111110
        db %11111110
        db %11111100
        db %11110000
        ds 48,0

PlayMus:
	ld a,(#2b)
	and 128
	jp nz,MusInt
	ld a,(muscnt)
	inc a
	ld (muscnt),a
	cp 6
	jp z,ResetMus
	jp MusInt
ResetMus:
	xor a
	ld (muscnt),a
	ret
MusInt:
           xor a
           ld (PT3_SETUP),a
           call	PT3_ROUT			; Write values on PSG registers
           call	PT3_PLAY			; Calculates PSG values for next frame
           call ayFX_PLAY
           ret

;Number in h to decimal ASCII
;Thanks to z80 Bits
;inputs:	hl = number to ASCII
DispA:
        ld h,0
        ld l,a
	ld	bc,-100
	call	NumA1
	ld	c,-10
	call	NumA1
	ld	c,-1
NumA1:	ld	a,'0'-1
NumA2:	inc	a
	add	hl,bc
	jr	c,NumA2
	sbc	hl,bc
	call outAtoVram;out (#98),a
	ret
